#!/usr/local/bin/wish 

option add *font -adobe-*-bold-r-*-*-20-*-75-75-*-*-*-* 
#option add *Background bisque

option add *activeBackground #ffffcf029580
option add *activeForeground black



#option add *selectBackground Skyblue
option add *selectBackground bisque
option add *selectBorderWidth 0c
option add *selectForeground Blue


option add *Scrollbar*background bisque
#option add *Scrollbar*Foreground burlywood
option add *Scrollbar*Foreground bisque
option add *Scrollbar*activeForeground #ffffcf029580



#/*------------------------------------------------------------------------*/
#/* Demo Initializations                                                   */
#/*------------------------------------------------------------------------*/

proc InitDemo {window_title} {
#ifndef LITTLE_SCREEN
    wm geometry . 500x400+5+60
#else /* LITTLE_SCREEN */
    wm geometry . 500x400+5+15
#endif /* LITTLE_SCREEN */
    wm title . $window_title

    frame .gt -borderwidth 4 -relief raised
    .gt configure
    pack  .gt    -side top -fill x
    label .gt.l1 -text "Daniel Diaz"
    label .gt.l2 -text "The clp(FD) System"
    label .gt.l3 -text "   INRIA"
    pack  .gt.l1 -side left
    pack  .gt.l3 -side right
    pack  .gt.l2 -anchor c 
    frame .title
    frame .body
    frame .menu
    label .null
}




#/*------------------------------------------------------------------------*/
#/* General Title / Body / Menu Managements                                */
#/*------------------------------------------------------------------------*/

proc Title {title_lst} {
    destroy .title
    frame   .title -relief groove -borderwidth 4
    pack    .title -side top -pady 25
    WriteText .title $title_lst
}



proc Body {body_lst} {
    destroy .body 
    frame .body
    pack  .body -side top
    WriteText .body $body_lst
}



proc Menu {menu_lst} {
    destroy  .menu
    frame .menu -borderwidth 4 -relief raised
    pack  .menu -side bottom -fill both
    set i 0
    foreach j $menu_lst {
	set name [lindex $j 0]
	set cmd [lindex $j 1]
	button  .menu.b$i -text $name -command $cmd
	pack .menu.b$i -side left -expand true -fill x
	incr i
    }
}




#/*------------------------------------------------------------------------*/
#/* General Text Writing                                                   */
#/*------------------------------------------------------------------------*/

proc WriteText {path text_lst} {
    global ln
    foreach i $text_lst {
	label $path.$ln -text "$i"
	pack $path.$ln -side top
	incr ln
    }
}




#/*------------------------------------------------------------------------*/
#/* Display File Procedure                                                 */
#/*------------------------------------------------------------------------*/

proc DisplayFile {file} {
    set browse [CreateTopLevel $file]
#ifndef LITTLE_SCREEN
    wm geometry .$browse +260+60
#else /* LITTLE_SCREEN */
    wm geometry .$browse +260+15
#endif /* LITTLE_SCREEN */

    frame .$browse.menu -borderwidth 4 -relief raised
    pack  .$browse.menu -side bottom -fill both
    button .$browse.menu.quit -text Quit -command "destroy .$browse" \
	    -relief raised
    pack .$browse.menu.quit -side bottom -fill x

    scrollbar .$browse.scroll -command ".$browse.list yview"
    pack .$browse.scroll -side right -fill y
    listbox .$browse.list -yscroll ".$browse.scroll set" -relief raised \
#ifndef LITTLE_SCREEN
	    -geometry 55x35 -setgrid yes
#else /* LITTLE_SCREEN */
	    -geometry 55x32 -setgrid yes
#endif /* LITTLE_SCREEN */
    pack .$browse.list -side left -fill both -expand yes

    set f [open $file r]
    while {[gets $f line] >= 0} {
	.$browse.list insert end [UnTabbify $line]
    }
    close $f
}



proc UnTabbify {line} {
    set p [string first \t $line]
    if {$p== -1} {
	return $line
    } else {
	set k [expr 8-(($p+1) % 8)+1]
	set s1 [string range $line 0 [expr $p-1]]
	set s2 [format "%*.*s" $k $k "" ]
	set s3 [string range $line [expr $p+1] 128]
	return [UnTabbify $s1$s2$s3]
    }
}




#/*------------------------------------------------------------------------*/
#/* New Top Level Name Generator                                           */
#/*------------------------------------------------------------------------*/

# global counters for top levels and lines in label widgets

set tl 0
set tl_pid(0) 0

set ln 0



proc CreateTopLevel {title} {
    global tl tl_pid

    incr tl
    set tl_pid($tl) 0
    toplevel .$tl
    wm title .$tl $title

    return $tl
}



proc DeleteTopLevel {toplev} {
    global tl_pid

    if {$tl_pid($toplev)!=0} {
	catch {exec kill $tl_pid($toplev)}
    }
    catch {destroy .$toplev}
}



#/*------------------------------------------------------------------------*/
#/* Prolog list to Tcl list converter                                      */
#/*------------------------------------------------------------------------*/

proc PrologListToTCL {list} {
    regsub -all {\[} $list \{ list1
    regsub -all {\]} $list1 \} list2
    regsub -all {,} $list2 \  list3
    return [lindex $list3 0]
}



#/*------------------------------------------------------------------------*/
#/* The Main Menu                                                          */
#/*------------------------------------------------------------------------*/

set main_title {
    "Presentation of clp(FD)" }


set main_body {
    "clp(FD) is a very efficient language for"
    "solving constraint-based problems"
    "(planning, scheduling, optimization,...)."
    ""
    "clp(FD) is developed at INRIA France"
    "(Research Institute for Computer Science)"
    ""
    "Choose a Demo..." }


set main_menu {
    { "Send"   SendMenu }
    { "Queens" QueensMenu }
    { "Square" SquareMenu }
    { "Bridge" BridgeMenu }
    { "Quit"   exit } }



proc MainMenu {} {
    global tl

    while {$tl>0} {
	DeleteTopLevel $tl
	incr tl -1
    }

    set ln 0
    global main_title main_body main_menu
    Title  $main_title
    Body   $main_body
    Menu   $main_menu
}




#/*------------------------------------------------------------------------*/
#/* Detecting when no more sol exists                                      */
#/*------------------------------------------------------------------------*/

proc DetectNoMoreSol {line} {
    string compare $line "No more solutions"
}




#/*------------------------------------------------------------------------*/
#/* The SEND Menu                                                          */
#/*------------------------------------------------------------------------*/

set send_title {
    "SEND + MORE = MONEY"
    "A Crypto-Arithmetic Problem" }

set send_body {
    "Find an assignment of digits to letters"
    "S,E,N,D,M,O,R,Y such that:"
    " "
    "    S E N D"
    "+   M O R E"
    "-----------"
    "= M O N E Y" }

set send_menu {
    { "Solve"        SolveSend}
    { "View Program" "DisplayFile send.pl" }
    { "Main Menu"    MainMenu } }



proc SendMenu {} {
    global send_title send_body send_menu
    Title  $send_title
    Body   $send_body
    Menu   $send_menu
}



proc SolveSend {} {
    set toplev [CreateSolveTopLevel "Send+More=Money"]
#ifndef LITTLE_SCREEN
    wm geometry .$toplev 400x280+408+195 
#else /* LITTLE_SCREEN */
    wm geometry .$toplev 400x280+408+150
#endif /* LITTLE_SCREEN */

    SolvePb $toplev "./send" DispSolveSend
}



proc DispSolveSend {toplev pipe} {
    gets $pipe w
    gets $pipe time
    set more [DetectNoMoreSol $w]
    if {$more} {
	scan $w "\[%d,%d,%d,%d,%d,%d,%d,%d\]" s e n d m o r y
	set w [list    "" \
                    "  S=$s E=$e N=$n D=$d M=$m O=$o R=$r Y=$y  " \
		       ""                 \
		       "    $s $e $n $d"  \
		       "+   $m $o $r $e"  \
		       "-----------"      \
		       "= $m $o $n $e $y" ]
    } else {
	set w [list "" "" $w]
    }

    SolveReInitBody $toplev
    WriteText .$toplev.body $w
    .$toplev.t configure -text $time

    return $more
}





#/*------------------------------------------------------------------------*/
#/* The QUEENS Menu                                                        */
#/*------------------------------------------------------------------------*/

set queens_title {
    "The N-Queens Problem" }

set queens_body {
    "Place N queens on an NxN chessboard"
    "so that no two queens attack each other"
    "" }

set queens_menu {
    { "Solve"        SolveQueens }
    { "View Program" "DisplayFile queens.pl" }
    { "Main Menu"    MainMenu } }



set N 4
set queens_info(1,1) 0



proc QueensMenu {} {
    global queens_title queens_body queens_menu
    Title  $queens_title
    Body   $queens_body

    global N
    set N 4
    frame .body.f
    pack  .body.f -side top
    label .body.f.l -text "Size: "
    entry .body.f.e  -relief sunken -width 3 -textvariable N
    pack  .body.f.l .body.f.e -side left
    bind  .body.f.e <Return> SolveQueens
    Menu  $queens_menu
}



proc SolveQueens {} {
    global N
    global queens_info

    set toplev [CreateSolveTopLevel "$N-Queens"]

#ifndef LITTLE_SCREEN
    set size [expr 640 / $N]
#else /* LITTLE_SCREEN */
    set size [expr 580 / $N]
#endif /* LITTLE_SCREEN */

#ifdef QUEENS_MIN
    if {$size<10} {
	set size 10
    }
#endif

    set x [expr $size*$N+41]
    set y [expr $x+80]

#ifndef LITTLE_SCREEN
    wm geometry .$toplev ${x}x$y+[expr 1100-$x]+100
#else /* LITTLE_SCREEN */
    wm geometry .$toplev ${x}x$y+[expr 900-$x]+20
#endif /* LITTLE_SCREEN */

    set queens_info($toplev,n) $N
    set queens_info($toplev,size) $size

    SolvePb $toplev "./queens $N" DispSolveQueens
}



proc DispSolveQueens {toplev pipe} {
    global queens_info

    set n    $queens_info($toplev,n)
    set size $queens_info($toplev,size)
    gets $pipe w
    gets $pipe time
    set more [DetectNoMoreSol $w]
    if {$more} {
	set pos [PrologListToTCL $w]
	puts stdout $pos
	DrawChessBoard $n $toplev $size
	DrawQueens     $n $toplev $size $pos
    } else {
	catch {destroy .$toplev.chess}
	set w [list "" "" "" "" "" "" $w]
	WriteText .$toplev.body $w
    }
    .$toplev.t configure -text $time

    return $more
}



proc DrawChessBoard {n toplev size} {
    set tot_sz [expr $n*$size]
    catch {destroy .$toplev.chess}
    canvas .$toplev.chess -width [expr $tot_sz+1] -height [expr $tot_sz+1] \
	    -bg snow
    pack .$toplev.chess -padx 20 -pady 20
    .$toplev.chess create rectangle 0 0 $tot_sz $tot_sz
    set size2 [expr $size*2]
    for {set i 0;set x 0} {$i<$n} {incr i;set x $x1} {
	set x1 [expr $x+$size]
        for {set j [expr $i%2];set y [expr $j*$size]} {$j<$n} \
	    {incr j 2;incr y $size2} {
	    set y1 [expr $y+$size]
            .$toplev.chess create rectangle $x $y $x1 $y1 -fill grey
        }
    }
}



proc DrawQueens {n toplev size pos} {
    for {set i 0;set x 2} {$i<$n} {incr i;incr x $size} {
        set y [expr ([lindex $pos $i]-1)*$size+2]
        .$toplev.chess create oval \
                   $x $y [expr $x+$size-4] [expr $y+$size-4] -fill SteelBlue1

    }
}




#/*------------------------------------------------------------------------*/
#/* The SQUARE Menu                                                        */
#/*------------------------------------------------------------------------*/

set square_title {
    "The Perfect Square Problem" }

set square_body {
    "Pack squares of given sizes into a"
    "large square in such a way that none"
    "of them overlap any other" 
    "" }

set square_menu {
    { "Data+Solve"   Data+SolveSquare }
    { "View Program" "DisplayFile square.pl" }
    { "Main Menu"    MainMenu } }



# data from square.pl

set PB 1

set square_data(1,size_x)  128
set square_data(1,size_y)  132
set square_data(1,size_sq) {72 60 56 40 36 32 28 16 4}

set square_data(2,size_x)  65
set square_data(2,size_y)  47
set square_data(2,size_sq) {25 24 23 22 19 17 11 6 5 3}

set square_data(3,size_x)  112
set square_data(3,size_y)  112
set square_data(3,size_sq) {50 42 37 35 33 29 27 25 24 19 18
                            17 16 15 11 9 8 7 6 4 2}

set square_data(4,size_x)  175
set square_data(4,size_y)  175
set square_data(4,size_sq) {81 64 56 55 51 43 39 38 35 33 31
                            30 29 20 18 16 14 9 8 5 4 3 2 1}


set square_info(1,1) 1


set colors {aquamarine1  grey SlateBlue #f50affff24dd CornflowerBlue 
            IndianRed1 DeepSkyBlue
            SandyBrown  PaleGreen  
            LightCoral pink GreenYellow LightSalmon 
            MediumOrchid lavender MediumSeaGreen chocolate4 cyan green 
            PaleTurquoise black orange tomato violet}
            
            


proc SquareMenu {} {
    global square_title square_body square_menu
    Title  $square_title
    Body   $square_body

    global PB
    set PB 1
    frame .body.f
    pack  .body.f -side top
    label .body.f.l -text "Problem Number (1/2/3): "
    entry .body.f.e  -relief sunken -width 3 -textvariable PB
    pack  .body.f.l .body.f.e -side left
    bind  .body.f.e <Return> Data+SolveSquare
    Menu  $square_menu
}



proc Data+SolveSquare {} {
    global PB
    global square_data square_info colors
    
    set toplev [CreateSolveTopLevel "Square (#$PB) Data"]
    destroy .$toplev.body

    # max_x  total size of data window 
    # (work space for all squares + square to fill)

#ifndef LITTLE_SCREEN
    set max_x 1100
#else /* LITTLE_SCREEN */
    set max_x 1000
#endif /* LITTLE_SCREEN */

    set nb_sq   [llength $square_data($PB,size_sq)]
    set size_x  $square_data($PB,size_x)
    set size_y  $square_data($PB,size_y)
    set size_sq $square_data($PB,size_sq)

#ifndef LITTLE_SCREEN
    set pt_sz [expr 500 / $size_x]
#else /* LITTLE_SCREEN */
    set pt_sz [expr 420 / $size_x]
#endif /* LITTLE_SCREEN */
    
    set sz_x [expr $size_x*$pt_sz]
    set sz_y [expr $size_y*$pt_sz]
    
    set square_info($toplev,pb)    $PB
    set square_info($toplev,nb_sq) [llength $square_data($PB,size_sq)]
    set square_info($toplev,pt_sz) $pt_sz
    set square_info($toplev,sz_x)  $sz_x
    set square_info($toplev,sz_y)  $sz_y

    set sz_work_x [expr $max_x - $sz_x - 40]

    canvas .$toplev.square -width  [expr $sz_x+1] -height [expr $sz_y+1]
    pack .$toplev.square -padx 20 -pady 20 -side right -anchor ne

    canvas .$toplev.work -width $sz_work_x -height 900
    pack .$toplev.work -side left -anchor ne
    .$toplev.square create rectangle 0 0 $sz_x $sz_y  -fill white

    set x 0
    set y 0
    set max_x_used 0

    for {set i 0} {$i<$nb_sq} {incr i} {
	set sz [expr [lindex $size_sq $i]*$pt_sz]
	if {$i==0 || [expr 20+$x+$sz]>$sz_work_x} {
	    set y [expr $y+$sz+20]
	    set x 20
	}  
	set x1   [expr $x+$sz]
	set y1   [expr $y-$sz]
	.$toplev.work create rectangle $x $y1 $x1 $y \
		-fill [lindex $colors $i] 
	incr x [expr 20+$sz]

	# compute max x used to adjust geometry at the end

	if {$x>$max_x_used} {
	    set max_x_used $x
	}
    }
    
    # compute necessary y size (without counting quit button see +34 below)

    incr y 20
    if {$y<[expr $sz_y+21]} {
	set y [expr $sz_y+41]
    }
    
    set x [expr $max_x_used+$sz_x+22]
    set y [expr $y+78]
#ifndef LITTLE_SCREEN
    wm geometry .$toplev ${x}x$y+[expr 1150-$x]+150
#else /* LITTLE_SCREEN */
    wm geometry .$toplev ${x}x$y+[expr 1000-$x]+40
#endif /* LITTLE_SCREEN */

    set cmd "./square $PB"
    .$toplev.menu.action configure -text Solve \
         -command [list SolvePb $toplev $cmd DispSolveSquare]
}


proc DispSolveSquare {toplev pipe} {
    global square_data square_info

    set nb_sq $square_info($toplev,nb_sq)

    gets $pipe w
    set more [DetectNoMoreSol $w]
    if {$more} {
	set pos_x [PrologListToTCL $w]

	gets $pipe w
	set pos_y [PrologListToTCL $w]

	gets $pipe w

	gets $pipe time

	puts stdout $pos_x
	puts stdout $pos_y

	DrawSquare $toplev $pos_x $pos_y
    } else {
	gets $pipe time
	destroy .$toplev.square
	frame .$toplev.body
	pack  .$toplev.body
	set w [list "" "" "" "" "" "" $w]
	WriteText .$toplev.body $w
    }

    .$toplev.t configure -text $time

    return $more
}



proc DrawSquare {toplev pos_x pos_y} {
    global square_data square_info colors

    set pb      $square_info($toplev,pb)
    set nb_sq   $square_info($toplev,nb_sq)
    set pt_sz   $square_info($toplev,pt_sz)
    set sz_x    $square_info($toplev,sz_x)
    set sz_y    $square_info($toplev,sz_y)
    set size_sq $square_data($pb,size_sq)

    .$toplev.square create rectangle 0 0 $sz_x $sz_y
    for {set i 0} {$i<$nb_sq} {incr i} {
	set x    [expr [lindex $pos_x $i]*$pt_sz]
	set y    [expr [lindex $pos_y $i]*$pt_sz]
	set sz   [expr [lindex $size_sq $i]*$pt_sz]
	set x1   [expr $x+$sz]
	set y1   [expr $y+$sz]
        .$toplev.square create rectangle $x $y $x1 $y1 \
		-fill [lindex $colors $i]

    }
}



#/*------------------------------------------------------------------------*/
#/* The BRIDGE Menu                                                        */
#/*------------------------------------------------------------------------*/

set bridge_title {
    "The Bridge Scheduling Problem" }

set bridge_body {
    "Find a schedule that minimizes the"
    "time to build a five-segment bridge"
    "" }

set bridge_menu {
    { "Data"         DataBridge }
    { "Solve"        SolveBridge }
    { "View Program" "DisplayFile bridge.pl" }
    { "Main Menu"    MainMenu } }



proc BridgeMenu {} {
    global bridge_title bridge_body bridge_menu
    Title  $bridge_title
    Body   $bridge_body

    Menu  $bridge_menu
}



proc DataBridge {} {
    set toplev [CreateTopLevel "Bridge-Data"]

    label  .$toplev.t -text "The Bridge" 
    pack   .$toplev.t -side top -fill x

    canvas .$toplev.c -width 615 -height 281 
    pack   .$toplev.c -pady 5 -side top

    DrawBridge $toplev

    frame .$toplev.menu -borderwidth 4 -relief raised
    pack  .$toplev.menu -side bottom -fill x

    button .$toplev.menu.quit -text Quit -command "destroy .$toplev"
    pack   .$toplev.menu.quit -side bottom -fill x -expand true


    frame .$toplev.brw
    pack  .$toplev.brw -side bottom -fill both -expand true

    frame .$toplev.brw.l
    pack  .$toplev.brw.l -side top -fill x
    label .$toplev.brw.l.nam -text Task -width 6
    label .$toplev.brw.l.des -text Description -width 31
    label .$toplev.brw.l.dur -text Durat
    label .$toplev.brw.l.res -text Resource 

    pack  .$toplev.brw.l.nam .$toplev.brw.l.des \
          .$toplev.brw.l.dur .$toplev.brw.l.res -side left -padx 1

    listbox .$toplev.brw.nam -yscroll "ListToScrollbar $toplev"\
#ifndef LITTLE_SCREEN
            -relief sunken -geometry 6x20 -setgrid yes
#else /* LITTLE_SCREEN */
            -relief sunken -geometry 6x15 -setgrid yes
#endif /* LITTLE_SCREEN */
    listbox .$toplev.brw.des -yscroll "ListToScrollbar $toplev"\
#ifndef LITTLE_SCREEN
            -relief sunken -geometry 32x20 -setgrid yes
#else /* LITTLE_SCREEN */
            -relief sunken -geometry 32x15 -setgrid yes
#endif /* LITTLE_SCREEN */
    listbox .$toplev.brw.dur -yscroll "ListToScrollbar $toplev"\
#ifndef LITTLE_SCREEN
            -relief sunken -geometry 3x20 -setgrid yes
#else /* LITTLE_SCREEN */
            -relief sunken -geometry 3x15 -setgrid yes
#endif /* LITTLE_SCREEN */
    listbox .$toplev.brw.res -yscroll "ListToScrollbar $toplev"\
#ifndef LITTLE_SCREEN
            -relief sunken -geometry 12x20 -setgrid yes
#else /* LITTLE_SCREEN */
            -relief sunken -geometry 12x15 -setgrid yes
#endif /* LITTLE_SCREEN */
    scrollbar .$toplev.brw.scroll \
	    -command "BridgeScrolling $toplev"

    pack .$toplev.brw.nam .$toplev.brw.des .$toplev.brw.dur \
         .$toplev.brw.res .$toplev.brw.scroll  -side left -fill y -padx 1

#ifndef LITTLE_SCREEN
    wm geometry .$toplev +15+70
#else /* LITTLE_SCREEN */
    wm geometry .$toplev +15+25
#endif /* LITTLE_SCREEN */

    set f [open bridge.data r]
    while {[gets $f line] >= 0} {
	set w   [split $line :]
	set nam [string trim [lindex $w 0]]
	set des [string trim [lindex $w 1]]
	set dur [lindex $w 2]
	set res [string trim [lindex $w 3]]
        .$toplev.brw.nam insert end $nam
        .$toplev.brw.des insert end $des
        .$toplev.brw.dur insert end $dur
        .$toplev.brw.res insert end $res
    }
    close $f
    

}



proc BridgeScrolling {toplev n} {
    .$toplev.brw.nam yview $n
    .$toplev.brw.des yview $n
    .$toplev.brw.dur yview $n
    .$toplev.brw.res yview $n
}



proc ListToScrollbar {toplev total_unit window_units first_unit last_unit} {
    .$toplev.brw.scroll set $total_unit $window_units $first_unit $last_unit
    .$toplev.brw.nam yview $first_unit
    .$toplev.brw.des yview $first_unit
    .$toplev.brw.dur yview $first_unit
    .$toplev.brw.res yview $first_unit
}



set pert_line(1)  "Import. dates"
set pert_line(2)  "Tem. housing"
set pert_line(3)  "Filling 1"
set pert_line(4)  "Pilar 1"
set pert_line(5)  "Bearer 1"
set pert_line(6)  "Pilar 2"
set pert_line(7)  "Bearer 2"
set pert_line(8)  "Pilar 3"
set pert_line(9)  "Bearer 3"
set pert_line(10) "Pilar 4"
set pert_line(11) "Bearer 4"
set pert_line(12) "Pilar 5"
set pert_line(13) "Bearer 5"
set pert_line(14) "Pilar 6"
set pert_line(15) "Filling 2"

set start_line_pilar  4

set step_x  5
set start_x 90

#ifndef LITTLE_SCREEN
set step_y  25
#else /* LITTLE_SCREEN */
set step_y  21
#endif /* LITTLE_SCREEN */
set start_y [expr (15+1)*$step_y]

set anim_pause(0)   0
set anim_line(0)    0
set anim_timming(0) 0


proc SolveBridge {} {
    global pert_line step_x start_x step_y start_y

    set toplev [CreateTopLevel Bridge]

#ifndef LITTLE_SCREEN
    wm geometry .$toplev +493+62
#else /* LITTLE_SCREEN */
    wm geometry .$toplev +360+0
#endif /* LITTLE_SCREEN */

    frame  .$toplev.menu -borderwidth 4 -relief raised
    pack   .$toplev.menu -side bottom -fill x
    frame  .$toplev.line
    pack   .$toplev.line -side bottom -pady 2
    label  .$toplev.line.t
    pack   .$toplev.line.t

    button .$toplev.menu.action -text Solve \
	    -command "DispSolveBridge $toplev"
    button .$toplev.menu.quit -text Quit -command "destroy .$toplev" 
    pack   .$toplev.menu.action .$toplev.menu.quit -side left -fill x \
	   -expand true

    # bridge

    canvas .$toplev.c -width 615 -height 281 
#ifndef LITTLE_SCREEN
    pack   .$toplev.c -padx 20 -pady 10 -side top
#else /* LITTLE_SCREEN */
    pack   .$toplev.c -padx 20 -pady 2 -side top
#endif /* LITTLE_SCREEN */

    DrawUnderGround $toplev MediumSeaGreen

    # pert

    canvas .$toplev.pert -width  [expr $start_x+105*$step_x+30] \
	                 -height [expr $start_y+40] \
			 -relief sunken -bg lavender
    pack   .$toplev.pert -side bottom

    for {set i 0;set x $start_x} {$i<=105} {incr i;incr x 5} {
	if {$i%5==0} {
	    .$toplev.pert create line $x $start_y $x [expr $start_y+10]
	    .$toplev.pert create text $x [expr $start_y+20] -text $i
	} else {
	    .$toplev.pert create line $x $start_y $x [expr $start_y+5]
	}
    }
    .$toplev.pert create line $start_x $start_y $x $start_y 


    set y $start_y
    for {set i 1} {$i<=15} {incr i} {
	incr y -$step_y
	.$toplev.pert create text 0 $y -text $pert_line($i) \
		-justify right -width [expr 2*$start_x-10]

	.$toplev.pert create line $start_x $y $x $y -stipple gray50
    }
}



proc DispSolveBridge {toplev} {
    global pert_line step_x start_x step_y start_y
    global col
    global anim_line

    exec >tmp bridge
    set f [open tmp r]

    gets $f w
    gets $f optim_end
    gets $f time

    puts $w
    set w [PrologListToTCL $w]

    set y [expr 470-3]

    set solut ""

    for {set i 0} {$i<46} {incr i} {
	set task_line [lindex $w $i]
	set task  [lindex $task_line 0]
	set durat [lindex $task_line 1]
	set start [lindex $task_line 2]

	set task  [string toupper $task]

	regsub {[1-9]}  $task "" prefix
	regsub {[A-Z]*} $task "" number

	if {[string length $number]==0} {
	    set number 0
	}

	set color $col($prefix)

	set line [LineOfTask $prefix $number]

	set x  [expr $start_x+$start*$step_x]
	set x1 [expr $x+$durat*$step_x]

	set y  [expr $start_y-($line*$step_y)-5]
	set y1 [expr $y+10]

	set end   [expr $start+$durat]

	if {$line>2 && [string compare $prefix C]!=0} {
	    set solut [concat $solut \
		    [list "$prefix $number $start $end $durat $line"]]
	}

	.$toplev.pert create rectangle $x $y $x1 $y1 -fill $color

	set x [expr ($x+$x1)/2]
	if {[regexp {B[1-6]} $task]} {
	    set y [expr $y-7]
	} else {
	    set y [expr $y1+8]
	}
	.$toplev.pert create text $x $y -text $task


    }
   
    set text_line "$time    Optimal end : $optim_end"

    DrawBridge $toplev

    # first (dummy) line for the animation

    set anim_line($toplev) [.$toplev.pert create line 0 0 0 0]

    .$toplev.line.t configure -text $text_line

    .$toplev.menu.action configure -text Animate \
	    -command "AnimBridge $toplev [list $solut] $optim_end [list $text_line]"
}



proc LineOfTask {prefix number} {
    set line 1

    if {[string compare [string index $prefix 0] U]==0} {
	set line 2
    }
	    
    if {[string compare $prefix V]==0} {
	if {$number==1} {
	    set line 3
	} else {
	    set line 15
	}
	
    } elseif {$number>0 && [string compare $prefix K]!=0} {
	if {[string compare $prefix P]==0} {
	    incr number 2
	}
	set line [expr $number*2+2]
	
	if {[string compare $prefix T]==0} {
	    incr line 1
	}
    }

 return $line
}




proc AnimBridge {toplev solut optim_end text_line} {
    global anim_line anim_timming loop

    destroy .$toplev.line.t
    frame   .$toplev.line.speed 

    label   .$toplev.line.speed.l -text Speed -width 10

    set anim_timming($toplev) 400

    radiobutton .$toplev.line.speed.s -variable anim_timming($toplev) \
	    -text Slow   -value 800
    radiobutton .$toplev.line.speed.m -variable anim_timming($toplev) \
	    -text Medium -value 400
    radiobutton .$toplev.line.speed.f -variable anim_timming($toplev) \
	    -text Fast   -value 0 


    pack .$toplev.line.speed -side bottom -pady 2
    pack .$toplev.line.speed.l .$toplev.line.speed.s \
	   .$toplev.line.speed.m .$toplev.line.speed.f   -side left -pady 2

    set loop 1

    checkbutton .$toplev.line.speed.loop -variable loop -text Loop
    pack .$toplev.line.speed.loop -side right -pady 2 -padx 50

    while {$loop} {

	destroy .$toplev.c 
	canvas  .$toplev.c -width 615 -height 281 

#ifndef LITTLE_SCREEN
        pack    .$toplev.c -padx 20 -pady 10 -side top
#else /* LITTLE_SCREEN */
        pack    .$toplev.c -padx 20 -pady 2 -side top
#endif /* LITTLE_SCREEN */

        DrawUnderGround $toplev MediumSeaGreen


        .$toplev.menu.action configure -text "Pause" \
		-command "set anim_pause($toplev) 1"

        # if Quit when the animation is running an errors occurs in AnimBridgeIter
        # when destroying .$toplev.anim_line($oplev)
        # So we use catch to avoid this error.
        # If no error then re-arm Animation button

        if {[catch {AnimBridgeIter $toplev $solut $optim_end}]==0} {
	    if {$loop==0} {

		destroy .$toplev.line.speed
		label  .$toplev.line.t
		pack   .$toplev.line.t -side bottom -pady 2

		.$toplev.line.t configure -text $text_line

		.$toplev.menu.action configure -text Animate \
			-command "AnimBridge $toplev [list $solut] $optim_end [list $text_line]"
	    } else {
		after 2000
	    }
	} else {
	    set loop 0
	}
    }
}



proc AnimBridgeIter {toplev solut optim_end} {
    global pert_line step_x start_x step_y start_y
    global col
    global anim_pause anim_timming anim_line

    set nb_tasks [llength $solut]
    set anim_pause($toplev) 0

    for {set i 0} {$i<=$optim_end} {incr i} {

	# manage pause/play buttons

	if {$anim_pause($toplev)} {
	    .$toplev.menu.action configure -text "Play " \
		    -command "set anim_pause($toplev) 0"
	    
	    while {$anim_pause($toplev)} {
		update 
	    }
	    after 100
	    
	    .$toplev.menu.action configure -text "Pause" \
		    -command "set anim_pause($toplev) 1"
	}

	.$toplev.pert delete $anim_line($toplev)

	set x [expr $start_x+$i*$step_x]
	set anim_line($toplev) [.$toplev.pert create line $x 0 $x $start_y]


	for {set t 0} {$t<$nb_tasks} {incr t} {
	    set task_line [lindex $solut $t]
	    set prefix [lindex $task_line 0]
	    set number [lindex $task_line 1]
	    set start  [lindex $task_line 2]
	    set end    [lindex $task_line 3]
	    set durat  [lindex $task_line 4]
	    set line   [lindex $task_line 5]
	    
	    if {$start<=$i && $i<=$end} {
		set pct [expr ($i-$start)*100/$durat]

		if {$pct==100} {
		    set color $col($prefix)
		} else {
		    set color grey$pct
		}
		
		DrawTask_$prefix $toplev $number $color
	    }
	}

	after $anim_timming($toplev)
	update

    }
}



# BRIDGE DRAWING
#
# coords: from 0 to 614 (symetrical axis at 307)
#
# y: from 0 to 320
# important values (b=base, t=top):
#         0: t of V1/V2 and T1/.../T5
#        28: b of T1/.../T5
#            t of M1/.../M6
#        77: b of M1/M6
#            t of A1/A6, S1/S6 and B1/B6
#       112: b of A1/A6, S1/S6 and B1/B6
#       150: b of M2/.../M5
#            t of A2/.../A5, S2/.../S5, B2/.../B5
#       185: b of A2/.../A5, S2/.../S5, B2/.../B5
#            t of P1/P2
#       250: b of P1/P2
#
# x: from 0 to 612 (symetrical axis in 307)
# important values (l=left, r=right):
#         0: l of V1                   614: r of V6
#        42: l of A1                   572: l of A6
#        61: l of M1/B1                553: r of M6/B6
#        87: r of V1                   527: l of V6
#            l of T1 (see pilar below)      r of T5
#       112: r of M1/B1                502: l of M6/B6
#       120: r of S1                   494: l of S6
#   pilars:
#       length: 88 starting-ending at 87-175/175-263/263-351/351-439/439-527
#


            

set col(A)  bisque
set col(S)  azure3
set col(B)  SandyBrown
set col(M)  MediumTurquoise
set col(T)  CornflowerBlue
set col(V)  PaleGreen
set col(P)  GreenYellow

# other tasks (not drawen)

set col(START) black
set col(STOP)  black
set col(UE)    IndianRed1
set col(UA)    IndianRed1
set col(K)     black
set col(L)     LightCoral
set col(C)     black



proc DrawBridge {toplev} {
    global col

    DrawUnderGround $toplev MediumSeaGreen

    foreach i {1 2 3 4 5 6} {
	DrawTask_A $toplev $i $col(A)
	DrawTask_S $toplev $i $col(S)
	DrawTask_B $toplev $i $col(B)
	DrawTask_M $toplev $i $col(M)
    }
	
    foreach i {1 2 3 4 5} {
	DrawTask_T $toplev $i $col(T)
    }

    foreach i {1 2} {
	DrawTask_P $toplev $i $col(P)
	DrawTask_V $toplev $i $col(V)
    }
}



proc DrawUnderGround {t col} {
    PolygonWithBorder $t "0 0  87 0  120 112  132 150  482 150  527 0  614 0  614 280  0 280" $col
}



proc DrawTask_A {t n col} {
    if {$n==1} {
	.$t.c create polygon   0 0   87 0  120 112   61 112 -fill $col

	set arrow_x  61
	set arrow_y 112
    } elseif {$n==6} {
	.$t.c create polygon 614 0  527 0  494 112  553 112 -fill $col

	set arrow_x 553
	set arrow_y 112
    } else {
	set a [expr 175-31+88*($n-2)]
	set b [expr $a+62]
	set c [expr $b-5]
	set d [expr $a+5]

	.$t.c create polygon $a 150   $b 150   $c 185   $d 185 -fill $col  

	if {$n<4} {set arrow_x $d} else {set arrow_x $c}
	set arrow_y 185
    }

    if {$arrow_x<307} {
	set x  [expr $arrow_x-27]
	set tx [expr $arrow_x-34]
    } else {
	set x  [expr $arrow_x+27]
	set tx [expr $arrow_x+34]
    }
    set y [expr $arrow_y+27]
    .$t.c create text $tx [expr $y+8] -text A$n
    .$t.c create line $x $y $arrow_x $arrow_y -arrow last
}



proc DrawTask_S {t n col} {
    if {$n==1} {
	PolygonWithBorder $t { 42 77   61  77   61 112  112 112 \
                              112 77  120 112   61 112} $col
	set arrow_x  57
	set arrow_y  94
    } elseif {$n==6} {
	PolygonWithBorder $t {572 77  553  77  553 112  502 112 \
                              502 77  494 112  553 112} $col
	set arrow_x 557
	set arrow_y 94
    } else {
	set b [expr 175-21+88*($n-2)]
	set a [expr $b-10]
	set c [expr $b+42]
	set d [expr $c+10]
	set e [expr $c+5]
	set f [expr $b-5]
	PolygonWithBorder $t "$a 150  $b 150  $b 185  $c 185 \
                              $c 150  $d 150  $e 185  $f 185" $col
	if {$n<4} {set arrow_x [expr $b-3]} else {set arrow_x [expr $c+3]}
	set arrow_y 167
    }

    if {$arrow_x<307} {
	set x  [expr $arrow_x-15]
	set tx [expr $arrow_x-23]
    } else {
	set x  [expr $arrow_x+15]
	set tx [expr $arrow_x+23]
    }
    set y [expr $arrow_y+27]
    .$t.c create text $tx $arrow_y -text S$n
    .$t.c create line $x $arrow_y  $arrow_x $arrow_y -arrow last
}


proc DrawTask_B {t n col} {
    if {$n==1} {
	.$t.c create rectangle  61 77   112 112 -fill $col
	.$t.c create text  88 94 -text B1
    } elseif {$n==6} {
	.$t.c create rectangle 553 77   502 112 -fill $col
	.$t.c create text 526 94 -text B6
    } else {
	set a [expr 175-21+($n-2)*88]
	set b [expr $a+42]
	.$t.c create rectangle $a 150   $b 185 -fill $col
	.$t.c create text [expr $a+21] 169 -text B$n
    }
}



proc DrawTask_M {t n col} {
    if {$n==1} {
	PolygonWithBorder $t { 87 28   112 28   112 77   61 77} $col
	.$t.c create text  95 55 -text M1
    } elseif {$n==6} {
	PolygonWithBorder $t {527 28   502 28   502 77   553 77} $col
	.$t.c create text 519 55 -text M6
    } else {
	set a [expr 175-14+($n-2)*88]
	set b [expr $a+28]
	.$t.c create rectangle $a 28   $b 150 -fill $col
	.$t.c create text [expr $a+14] 89 -text M$n
    }
}



proc DrawTask_T {t n col} {
    set a [expr 87+($n-1)*88]
    set b [expr $a+88]
    .$t.c create rectangle $a 0   $b 28 -fill $col
    .$t.c create text [expr $a+44] 14 -text T$n
}



proc DrawTask_V {t n col} {
    if {$n==1} {
	PolygonWithBorder $t {  0 0   87 0   87 28   61 77   42 77} $col
	.$t.c create text  45 30 -text V1
    } else {
	PolygonWithBorder $t {614 0  527 0  527 28  553 77  572 77} $col
	.$t.c create text 569 30 -text V2
    }
}



proc DrawTask_P {t n col} {
    set a [expr 175+$n*88+1]
    set x [expr $a-21]
    set y 230
    
    for {set i 0} {$i<5} {incr i} {
	.$t.c create rectangle $x 185 [expr $x+5] $y -fill $col
	set x [expr $x+9]
	if {$i<2} {incr y 10} else {incr y -10}
    }
    .$t.c create text $a 265 -text P$n
}



proc PolygonWithBorder {toplev coords color} {
    eval .$toplev.c create polygon $coords -fill $color

    set coords [concat $coords [lrange $coords 0 1]]
    eval .$toplev.c create line $coords 
}



#/*------------------------------------------------------------------------*/
#/* General Solve Procedure                                                */
#/*------------------------------------------------------------------------*/

proc CreateSolveTopLevel {title} {
    set toplev [CreateTopLevel $title]

    frame  .$toplev.body
    pack   .$toplev.body -side top -fill x
    label  .$toplev.t
    frame  .$toplev.menu -borderwidth 4 -relief raised
    pack   .$toplev.menu -side bottom -fill x
    pack   .$toplev.t -side bottom -pady 9

    button .$toplev.menu.action
    button .$toplev.menu.quit -text Quit -command "DeleteTopLevel $toplev"
    pack   .$toplev.menu.action .$toplev.menu.quit -side left -fill x \
	   -expand true

    return $toplev
}



proc SolvePb {toplev command disp_proc} {
    global tl_pid

    set pipe [open "|$command" r+]
    set tl_pid($toplev) [pid $pipe]

    .$toplev.menu.action configure -text Next \
            -command "SolveNext $toplev $pipe $disp_proc"

    if {[$disp_proc $toplev $pipe]==0} {
	.$toplev.menu.action configure  -state disabled
	close $pipe
    }
}



proc SolveNext {toplev pipe disp_proc} {
    puts  $pipe c
    flush $pipe
    if {[$disp_proc $toplev $pipe]==0} {
	.$toplev.menu.action configure  -state disabled
	close $pipe
    }
}




proc SolveReInitBody {toplev} {
    destroy .$toplev.body
    frame   .$toplev.body
    pack    .$toplev.body -side top -fill x
}    




#/*------------------------------------------------------------------------*/
#/* MAIN                                                                   */
#/*------------------------------------------------------------------------*/

InitDemo "Demo"
MainMenu
