/*--------------------------------------------------*/
/* Benchmark clp(FD)             INRIA Rocquencourt */
/*                                                  */
/* Name  : square.pl                                */
/* Title : perfect square                           */
/*--------------------------------------------------*/
:- main([top]).


problem(1, 128,132,[72,60,56,40,36,32,28,16,4]).

problem(2, 65, 47,[25,24,23,22,19,17,11,6,5,3]).

problem(3,112,112,[50,42,37,35,33,29,27,25,24,19,18,
                   17,16,15,11,9,8,7,6,4,2]).

problem(4,175,175,[81,64,56,55,51,43,39,38,35,33,31,
                   30,29,20,18,16,14,9,8,5,4,3,2,1]).


square(P,Xs,Ys,Ss):-
	gen(P,Xs,Ys,Ss,SX,SY),
	(SX>=SY -> MaxS=SX
                ;  MaxS=SY),
	fd_vector_max(MaxS),
	no_overlap(Xs,Ys,Ss),
	cap(Xs,Ss,SX,SY),
	cap(Ys,Ss,SY,SX),
	label(Xs),
	label(Ys).


gen(P,Xs,Ys,Ss,SX,SY):-
	problem(P,SX,SY,Ss),
	gen_coords(Ss,Xs,Ys,SX,SY).


gen_coords([],[],[],_,_).

gen_coords([S|Ss],[X|Xs],[Y|Ys],SX,SY):-
	X in 0..SX-S,
	Y in 0..SY-S,
	gen_coords(Ss,Xs,Ys,SX,SY).


no_overlap([],[],[]).

no_overlap([X|Xs],[Y|Ys],[S|Ss]):-
	no_overlap1(Xs,Ys,Ss,X,Y,S),
	no_overlap(Xs,Ys,Ss).


no_overlap1([],[],[],_,_,_).

no_overlap1([X2|Xs],[Y2|Ys],[S2|Ss],X1,Y1,S1):-
	no_overlap2(X1,Y1,S1,X2,Y2,S2),
	no_overlap1(Xs,Ys,Ss,X1,Y1,S1).


	% X1+S1 #=< X2 \/ X1 #>= X2+S2 \/ 
        % Y1+S1 #=< Y2 \/ Y1 #>= Y2+S2

no_overlap2(X1,Y1,S1,X2,Y2,S2):-
	no_overlap_iff_b(X1,S1,X2,S2,B1),
	no_overlap_iff_b(Y1,S1,Y2,S2,B2),
	or1(B1,B2).


no_overlap_iff_b(X1,S1,X2,S2,B):-
	B  in 'X1_X2_To_B'(min(X1),max(X1),min(X2),max(X2),S1,S2),
	X1 in 'B_X2_To_X1'(val(B),S1,min(X2),max(X2),S2),
	X2 in 'B_X2_To_X1'(val(B),S2,min(X1),max(X1),S1).


cap(Xs,Ss,SX,SY):-
	cap1(0,SX,SY,Xs,Ss).


cap1(P,SX,SY,Xs,Ss):-
	(P < SX -> sum_of_squares_with(Xs,Ss,P,SY),
   	           P1 is P+1,
		   cap1(P1,SX,SY,Xs,Ss)
                ;
                   true).


sum_of_squares_with([],[],_,0).

sum_of_squares_with([X|Xs],[S|Ss],P,Sum):-
	point_used_by_square_iff_b(P,X,S,B),
	'ax+y=z'(S,B,Sum1,Sum),
	sum_of_squares_with(Xs,Ss,P,Sum1).


	% X<=P<X+S <=> B     P and S are ground

point_used_by_square_iff_b(P,X,S,B):-
	B in 'X_To_B'(min(X),max(X),P,S),
	X in 'B_To_X'(val(B),P-S+1..P).


label([]).

label([X|Xs]):-
	min_list([X|Xs],Min),
	select_square([X|Xs],Min,Rest),
	label(Rest).


min_list([X|Xs],Min):-
	fd_min(X,Min1),
	min_list1(Xs,Min1,Min).


min_list1([],M,M).

min_list1([X|Xs],M1,M):-
	fd_min(X,M2),
	(M1=<M2 -> M3=M1
                ;  M3=M2),
	min_list1(Xs,M3,M).


select_square([X|Xs],X,Xs).

select_square([X|Xs],Min,[X|Rest]):-
	X#>Min,
	select_square(Xs,Min,Rest).


:- public start/2.

start([Xs,Ys,Ss],[NX]):-
	number_atom(N,NX), 
	square(N,Xs,Ys,Ss).
